"""Module for representing CIViC assertion record as GKS AAC 2017 Study Statement"""

from abc import ABC
from enum import Enum
import re
from types import MappingProxyType

from ga4gh.cat_vrs.models import CategoricalVariant
from ga4gh.core.models import (
    Coding,
    ConceptMapping,
    Extension,
    MappableConcept,
    Relation,
)
from ga4gh.va_spec.aac_2017 import (
    Classification,
    Strength,
    VariantDiagnosticStudyStatement,
    VariantPrognosticStudyStatement,
    VariantTherapeuticResponseStudyStatement,
)
from ga4gh.va_spec.base import (
    Agent,
    Contribution,
    DiagnosticPredicate,
    Direction,
    Document,
    EvidenceLine,
    MembershipOperator,
    Method,
    PrognosticPredicate,
    Statement,
    System,
    TherapeuticResponsePredicate,
    TherapyGroup,
    VariantDiagnosticProposition,
    VariantPrognosticProposition,
    VariantTherapeuticResponseProposition,
)
from ga4gh.vrs.models import Expression, Syntax
from pydantic import PrivateAttr
from civicpy.civic import (
    Assertion,
    Coordinate,
    Endorsement,
    Evidence,
    Disease,
    Organization,
    Therapy,
    GeneVariant,
    MolecularProfile,
    get_gene_by_id,
)


class CivicGksRecordError(Exception):
    """Custom error for CIViC GKS Record exceptions"""


class CivicInteractionType(str, Enum):
    """Define constraints for the translation of supported CIViC interaction types into
    GKS

    SEQUENTIAL is not currently supported
    """

    SUBSTITUTES = "SUBSTITUTES"
    COMBINATION = "COMBINATION"


class CivicEvidenceAssertionType(str, Enum):
    """Define constraints for the translation of supported CIViC evidence and assertion
    types into GKS

    ONCOGENIC and PREDISPOSING are not currently supported
    """

    PREDICTIVE = "PREDICTIVE"
    PROGNOSTIC = "PROGNOSTIC"
    DIAGNOSTIC = "DIAGNOSTIC"


class CivicEvidenceLevel(str, Enum):
    """Define constraints for CIViC evidence levels"""

    A = "A"
    B = "B"
    C = "C"
    D = "D"
    E = "E"


class CivicEvidenceName(str, Enum):
    """Define constraints for CIViC evidence names"""

    VALIDATED_ASSOCIATION = "Validated association"
    CLINICAL_EVIDENCE = "Clinical evidence"
    CASE_STUDY = "Case study"
    PRECLINICAL_EVIDENCE = "Preclinical evidence"
    INFERENTIAL_ASSOCIATION = "Inferential association"


# CIViC evidence level to CIViC evidence level name
CIVIC_EVIDENCE_LEVEL_TO_NAME = MappingProxyType(
    {
        CivicEvidenceLevel.A: CivicEvidenceName.VALIDATED_ASSOCIATION,
        CivicEvidenceLevel.B: CivicEvidenceName.CLINICAL_EVIDENCE,
        CivicEvidenceLevel.C: CivicEvidenceName.CASE_STUDY,
        CivicEvidenceLevel.D: CivicEvidenceName.PRECLINICAL_EVIDENCE,
        CivicEvidenceLevel.E: CivicEvidenceName.INFERENTIAL_ASSOCIATION,
    }
)


# CIViC significance to GKS predicate
CLIN_SIG_TO_PREDICATE = MappingProxyType(
    {
        "SENSITIVITYRESPONSE": TherapeuticResponsePredicate.SENSITIVITY,
        "RESISTANCE": TherapeuticResponsePredicate.RESISTANCE,
        "POOR_OUTCOME": PrognosticPredicate.WORSE_OUTCOME,
        "BETTER_OUTCOME": PrognosticPredicate.BETTER_OUTCOME,
        "POSITIVE": DiagnosticPredicate.INCLUSIVE,
        "NEGATIVE": DiagnosticPredicate.EXCLUSIVE,
    }
)

# SNP pattern
_SNP_RE = re.compile(r"RS\d+")


class _CivicGksAssertionRecord(ABC):
    """Abstract class for CIViC assertion record represented as GKS

    :param assertion: CIViC assertion record
    :raises CivicGksRecordError: If CIViC assertion is not able to be represented as
        GKS object
    """

    _method: Method = PrivateAttr(
        Method(
            id="civic.method:2019",
            name="CIViC Curation SOP (2019)",
            reportedIn=Document(
                name="Danos et al., 2019, Genome Med.",
                title="Standard operating procedure for curation and clinical interpretation of variants in cancer",
                doi="10.1186/s13073-019-0687-x",
                pmid=31779674,
            ),
            methodType="variant curation standard operating procedure",
        )
    )
    _assertion: Assertion = PrivateAttr()

    def __init__(
        self,
        assertion: Assertion,
        endorsement: Endorsement | None = None,
    ) -> None:
        """Initialize _CivicGksAssertionRecord class

        :param assertion: CIViC assertion record
        :param endorsement: CIViC endorsement for the assertion, defaults to None
        :raises CivicGksRecordError: If CIViC assertion is not able to be represented as
            GKS object
        """
        object.__setattr__(self, "_assertion", assertion)

        if not self._assertion.is_valid_for_gks_json(emit_warnings=True):
            err_msg = "Assertion is not valid for GKS."
            raise CivicGksRecordError(err_msg)

        classification, strength = self.classification_and_strength(
            self._assertion.amp_level
        )

        if endorsement:
            organization = endorsement.organization
            contributions = [
                Contribution(
                    activityType=f"{endorsement.type}.last_reviewed",
                    date=endorsement.last_reviewed.split("T", 1)[0],
                    contributor=Agent(
                        id=f"civic.{organization.type}:{organization.id}",
                        name=organization.name,
                        description=organization.description,
                    ),
                )
            ]
        else:
            contributions = None

        super().__init__(
            id=f"civic.aid:{self._assertion.id}",
            contributions=contributions,
            description=self._assertion.description,
            specifiedBy=self.method(),
            proposition=self.proposition(self._assertion),
            direction=self.direction(self._assertion.assertion_direction),
            classification=classification,
            strength=strength,
            hasEvidenceLines=self.evidence_lines(),
        )

    def classification_and_strength(
        self,
        amp_level: str,
    ) -> tuple[MappableConcept | None, MappableConcept | None]:
        """Get classification and strength

        :param amp_level: AMP/ASCO/CAP level
        :return: Classification and strength, if found
        """
        classification = None
        strength = None
        system = System.AMP_ASCO_CAP

        if amp_level != "NA":
            pattern = re.compile(r"TIER_(?P<tier>[IV]+)(?:_LEVEL_(?P<level>[A-D]))?")
            match = pattern.match(amp_level).groupdict()
            classification = MappableConcept(
                primaryCoding=Coding(
                    code=Classification(f"Tier {match['tier']}"), system=system
                ),
            )

            level = match["level"]
            evidence_strength = self.evidence_strength(CivicEvidenceLevel(level))
            evidence_strength.primaryCoding.name = evidence_strength.name
            mappings = [
                ConceptMapping(
                    coding=evidence_strength.primaryCoding,
                    relation=Relation.EXACT_MATCH,
                )
            ]

            strength = MappableConcept(
                primaryCoding=Coding(code=Strength(f"Level {level}"), system=system),
                mappings=mappings,
            )

        return classification, strength

    def evidence_strength(self, evidence_level: CivicEvidenceLevel) -> MappableConcept:
        """Get CIViC Evidence Item strength

        :param evidence_level: CIViC evidence level
        :return: Strength for CIViC evidence item
        """
        return MappableConcept(
            name=CIVIC_EVIDENCE_LEVEL_TO_NAME[evidence_level],
            primaryCoding=Coding(
                system="https://civic.readthedocs.io/en/latest/model/evidence/level.html",
                code=evidence_level.value,
            ),
        )

    def method(self) -> Method:
        """Get GKS method

        :return: GKS method
        """
        return self._method.default

    def proposition(
        self, record: Evidence | Assertion
    ) -> (
        VariantTherapeuticResponseProposition
        | VariantDiagnosticProposition
        | VariantPrognosticProposition
    ):
        """Get GKS proposition

        :param record: CIViC assertion or evidence item
        :return: GKS proposition
        """
        variant: GeneVariant = record.variants[0]

        params = {
            "subjectVariant": self.variant(record.molecular_profile, variant),
            "geneContextQualifier": self.gene(variant.gene.id),
            "alleleOriginQualifier": self.allele_origin_qualifier(record),
            "predicate": self.predicate(record),
        }

        record_type = (
            record.assertion_type
            if isinstance(record, Assertion)
            else record.evidence_type
        )

        if record_type == CivicEvidenceAssertionType.PREDICTIVE:
            condition_key = "conditionQualifier"
            params["objectTherapeutic"] = self.therapeutic(
                record.therapies, self._assertion.therapy_interaction_type
            )
            proposition = VariantTherapeuticResponseProposition
        else:
            condition_key = "objectCondition"

            if record_type == CivicEvidenceAssertionType.PROGNOSTIC:
                proposition = VariantPrognosticProposition
            else:
                proposition = VariantDiagnosticProposition

        params[condition_key] = self.condition(record.disease)

        return proposition(**params)

    def direction(self, record_direction: str) -> Direction | None:
        """Get direction for CIViC assertion or evidence item

        :param record_direction: CIViC assertion or evidence item's direction
        :return: Direction for CIViC assertion or evidence item
        """
        if record_direction == "SUPPORTS":
            return Direction.SUPPORTS
        if record_direction == "DOES_NOT_SUPPORT":
            return Direction.DISPUTES
        return None

    def evidence_lines(self) -> list[EvidenceLine]:
        """Get evidence lines for a CIViC assertion

        Only the CIViC evidence items that are supported for GKS will be included

        :raises CivicGksRecordError: If evidence item is not valid for GKS
        :return: List of CIViC evidence lines
        """
        evidence_lines: list[EvidenceLine] = []

        for evidence_item in self._assertion.evidence_items:
            if not evidence_item.is_valid_for_gks_json(emit_warnings=True):
                err_msg = f"Evidence {evidence_item.id} is not valid for GKS."
                raise CivicGksRecordError(err_msg)

            ev_source = evidence_item.source

            evidence_lines.append(
                EvidenceLine(
                    hasEvidenceItems=[
                        Statement(
                            id=f"civic.eid:{evidence_item.id}",
                            description=evidence_item.description,
                            specifiedBy=self.method(),
                            proposition=self.proposition(evidence_item),
                            direction=self.direction(evidence_item.evidence_direction),
                            strength=self.evidence_strength(
                                CivicEvidenceLevel(evidence_item.evidence_level)
                            ),
                            reportedIn=[
                                Document(
                                    id=f"civic.source:{ev_source.id}",
                                    name=ev_source.citation,
                                    title=ev_source.title,
                                    pmid=int(ev_source.citation_id)
                                    if ev_source.source_type == "PUBMED"
                                    else None,
                                )
                            ],
                        )
                    ],
                    directionOfEvidenceProvided=Direction.SUPPORTS,
                )
            )

        return evidence_lines

    @staticmethod
    def variant(
        molecular_profile: MolecularProfile, variant: GeneVariant
    ) -> CategoricalVariant:
        """Get GKS CategoricalVariant

        :param molecular_profile: CIViC simple molecular profile
        :param variant: CIViC variant corresponding to ``molecular_profile``
        :return: GKS CategoricalVariant
        """
        extensions = [
            Extension(
                name="CIViC Molecular Profile Score",
                value=molecular_profile.molecular_profile_score,
            )
        ]
        if variant.hgvs_expressions:
            expressions = []

            for hgvs_expr in variant.hgvs_expressions:
                if hgvs_expr == "N/A":
                    continue

                if "p." in hgvs_expr:
                    syntax = Syntax.HGVS_P
                elif "c." in hgvs_expr:
                    syntax = Syntax.HGVS_C
                elif "g." in hgvs_expr:
                    syntax = Syntax.HGVS_G
                else:
                    continue

                expressions.append(Expression(syntax=syntax, value=hgvs_expr))

            if expressions:
                extensions.append(Extension(name="expressions", value=expressions))

        if isinstance(variant.coordinates, Coordinate):
            coords = variant.coordinates
            extensions.append(
                Extension(
                    name="CIViC representative coordinate",
                    value={
                        "chromosome": coords.chromosome,
                        "start": coords.start,
                        "stop": coords.stop,
                        "reference_bases": coords.reference_bases,
                        "variant_bases": coords.variant_bases,
                        "ensembl_version": coords.ensembl_version,
                        "representative_transcript": coords.representative_transcript,
                        "reference_build": coords.reference_build,
                        "type": coords.type,
                    },
                )
            )

        aliases = []
        mappings = []
        for a in molecular_profile.aliases:
            if _SNP_RE.match(a):
                a = a.lower()
                mappings.append(
                    ConceptMapping(
                        coding=Coding(
                            code=a,
                            system="https://www.ncbi.nlm.nih.gov/snp/",
                        ),
                        relation=Relation.RELATED_MATCH,
                    )
                )
            else:
                aliases.append(a)

        return CategoricalVariant(
            id=f"civic.mpid:{molecular_profile.id}",
            name=molecular_profile.name,
            description=molecular_profile.description,
            aliases=aliases or None,
            extensions=extensions,
            mappings=mappings or None,
        )

    @staticmethod
    def gene(gene_id: int) -> MappableConcept:
        """Get GKS gene

        :param gene_id: ID for CIViC gene
        :return: GKS gene
        """
        gene = get_gene_by_id(gene_id)

        if gene.aliases:
            extensions = [Extension(name="aliases", value=gene.aliases)]
        else:
            extensions = []

        if gene.description:
            extensions.append(Extension(name="description", value=gene.description))

        if gene.entrez_id:
            entrez_id = str(gene.entrez_id)
            mappings = [
                ConceptMapping(
                    coding=Coding(
                        id=f"ncbigene:{entrez_id}",
                        code=entrez_id,
                        system="https://www.ncbi.nlm.nih.gov/gene/",
                    ),
                    relation=Relation.EXACT_MATCH,
                )
            ]
        else:
            mappings = None

        return MappableConcept(
            id=f"civic.gid:{gene_id}",
            conceptType="Gene",
            name=gene.name,
            mappings=mappings,
            extensions=extensions or None,
        )

    @staticmethod
    def allele_origin_qualifier(record: Evidence | Assertion) -> MappableConcept:
        """Get GKS allele origin qualifier

        :param record: CIViC assertion or evidence item
        :return: Allele origin qualifier
        """
        return MappableConcept(name=record.variant_origin)

    @staticmethod
    def predicate(
        record: Evidence | Assertion,
    ) -> (
        PrognosticPredicate | DiagnosticPredicate | TherapeuticResponsePredicate | None
    ):
        """Get GKS predicate

        :param record: CIViC assertion or evidence item
        :raises CivicGksRecordError: If significance is not supported for GKS
        :return: GKS predicate
        """
        try:
            return CLIN_SIG_TO_PREDICATE[record.significance]
        except KeyError:
            err_msg = f"Significance is not supported for GKS: {record.significance}"
            raise CivicGksRecordError(err_msg)

    @staticmethod
    def therapeutic(
        therapies: list[Therapy], therapy_interaction_type: str | None
    ) -> MappableConcept | TherapyGroup | None:
        """Get GKS therapeutic or TherapyGroup

        :param therapies: List of CIViC therapy records
        :param therapy_interaction_type: The interaction type for ``therapies``
        :raises CivicGksRecordError: If no therapies are found
        :return: GKS therapeutic (single therapy record) or TherapyGroup (multiple
            therapies)
        """

        def _get_therapy(therapy: Therapy) -> MappableConcept:
            """Get GKS therapeutic

            :param therapy: CIViC therapy
            :return: GKS therapeutic
            """
            if therapy.aliases:
                extensions = [Extension(name="aliases", value=therapy.aliases)]
            else:
                extensions = None

            if therapy.ncit_id:
                mappings = [
                    ConceptMapping(
                        coding=Coding(
                            id=f"ncit:{therapy.ncit_id}",
                            code=therapy.ncit_id,
                            system="https://ncit.nci.nih.gov/ncitbrowser/ConceptReport.jsp?dictionary=NCI_Thesaurus&code=",
                        ),
                        relation=Relation.EXACT_MATCH,
                    )
                ]
            else:
                mappings = None

            return MappableConcept(
                id=f"civic.tid:{therapy.id}",
                name=therapy.name,
                conceptType="Therapy",
                mappings=mappings,
                extensions=extensions,
            )

        if not therapies:
            err_msg = "No therapies found"
            raise CivicGksRecordError(err_msg)

        if len(therapies) == 1:
            therapy: Therapy = therapies[0]
            return _get_therapy(therapy)
        else:
            membership_operator = (
                MembershipOperator.AND
                if therapy_interaction_type == CivicInteractionType.COMBINATION
                else MembershipOperator.OR
            )
            therapies_mc: list[MappableConcept] = [_get_therapy(t) for t in therapies]
            return TherapyGroup(
                therapies=therapies_mc, membershipOperator=membership_operator
            )

    @staticmethod
    def condition(disease: Disease) -> MappableConcept:
        """Get GKS condition

        :param disease: CIViC disease
        :return: GKS condition
        """
        if disease.doid:
            doid = f"DOID:{disease.doid}"
            mappings = [
                ConceptMapping(
                    coding=Coding(
                        id=doid,
                        code=doid,
                        system="https://disease-ontology.org/?id=",
                    ),
                    relation=Relation.EXACT_MATCH,
                )
            ]
        else:
            mappings = None

        return MappableConcept(
            id=f"civic.did:{disease.id}",
            conceptType="Disease",
            name=disease.name,
            mappings=mappings,
        )


class CivicGksPredictiveAssertion(
    _CivicGksAssertionRecord, VariantTherapeuticResponseStudyStatement
):
    """Class for representing CIViC predictive assertion as GKS VariantTherapeuticResponseStudyStatement"""


class CivicGksDiagnosticAssertion(
    _CivicGksAssertionRecord, VariantDiagnosticStudyStatement
):
    """Class for representing CIViC diagnostic assertion as GKS VariantDiagnosticStudyStatement"""


class CivicGksPrognosticAssertion(
    _CivicGksAssertionRecord, VariantPrognosticStudyStatement
):
    """Class for representing CIViC prognostic assertion as GKS VariantPrognosticStudyStatement"""



from setuptools import setup, find_packages
from civicpy.__version__ import __version__, __authors__, __author_email__, __description__, __url__
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='civicpy',
    version=__version__,
    packages=find_packages(),
    url=__url__,
    license='MIT',
    author=', '.join(__authors__),
    author_email=__author_email__,
    description=__description__,
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3'
    ],
    install_requires=[
        'requests',
        'obonet',
        'networkx',
        'pandas',
        'Click',
        'vcfpy',
        'pysam',
        'backports-datetime-fromisoformat',
        'deprecation',
        'ga4gh.vrs',
        'ga4gh.cat_vrs',
        'ga4gh.va_spec~=0.3.0'
    ],
    extras_require={
        'test': [
            'pytest==6.2.5',
            'pytest-cov==5.0.0',
            'attrs==22.1.0',
            'coveralls',
            'coverage<7.4.4',
            'deepdiff'
        ],
        'docs': [
            'sphinx',
            'sphinxjp.themes.basicstrap',
            'sphinxcontrib.programoutput'
        ]
    },
    python_requires='>=3.10',
    entry_points={
        'console_scripts': [
            'civicpy=civicpy.cli:cli'
        ]
    },
)



from .__version__ import __version__
from pathlib import Path
import os

REMOTE_CACHE_URL = os.getenv('CIVICPY_REMOTE_CACHE_URL', False) or \
    "https://civicdb.org/downloads/nightly/nightly-civicpy_cache.pkl"
LOCAL_CACHE_PATH = os.getenv('CIVICPY_CACHE_FILE', False) or \
    str(Path.home() / '.civicpy' / 'cache.pkl')
CACHE_TIMEOUT_DAYS = os.getenv('CIVICPY_CACHE_TIMEOUT_DAYS', False) or 7
# We convert the CACHE_TIMEOUT_DAYS to int
# if not possible we use the default value
try:
    CACHE_TIMEOUT_DAYS = int(CACHE_TIMEOUT_DAYS)
except TypeError:
    CACHE_TIMEOUT_DAYS = 7

TEST_CACHE_PATH = str(Path(__file__).parent / 'data' / 'test_cache.pkl')


def version():
    return __version__



import vcfpy
import requests
import re
from civicpy.civic import GeneVariant

class CivicVcfRecord(vcfpy.Record):
    """
    :param civic.GeneVariant variant: A :class:`civic.GeneVariant` object to convert to a :class:`civic.exports.CivicVcfRecord` object (inherits from vcfpy.Record)
    """

    def __init__(self, variant, include_status=["submitted", "accepted"]):
        if not isinstance(variant, GeneVariant):
            raise Exception('Variant is not a GeneVariant.')
        self.variant = variant

        if not self.variant.is_valid_for_vcf(emit_warnings=True):
            raise Exception('Variant is not valid for VCF.')

        if self.vcf_coordinates() is None:
            raise Exception("Variant doesn't have any coordinates.")

        (start, ref, alt) = self.vcf_coordinates()
        info = vcfpy.OrderedDict({
            ('GN', self.variant.gene.name),
            ('VT', self.sanitized_name()),
            ('CSQ', tuple(self.csq(include_status))),
        })

        super().__init__(
            CHROM=self.variant.coordinates.chromosome,
            POS=start,
            ID=[str(self.variant.id)],
            REF=ref,
            ALT=[self.alt_record(alt)],
            QUAL=None,
            FILTER=['.'],
            INFO=info,
        )

    def vcf_coordinates(self):
        ensembl_server = "https://grch37.rest.ensembl.org"
        if self.variant.coordinates.reference_build != 'GRCh37':
            return
        if self.variant.is_insertion:
            if not self.variant.coordinates.representative_transcript:
                return
            else:
                start = self.variant.coordinates.start
                ext = "/sequence/region/human/{}:{}-{}".format(self.variant.coordinates.chromosome, start, start)
                r = requests.get(ensembl_server+ext, headers={ "Content-Type" : "text/plain"})
                r.raise_for_status()
                if self.variant.coordinates.reference_bases == None or self.variant.coordinates.reference_bases == '-' or self.variant.coordinates.reference_bases == '':
                    ref = r.text
                else:
                    ref = "{}{}".format(r.text, self.variant.coordinates.reference_bases)
                alt = "{}{}".format(r.text, self.variant.coordinates.variant_bases)
        elif self.variant.is_deletion:
            if not self.variant.coordinates.representative_transcript:
                return
            else:
                start = self.variant.coordinates.start - 1
                ext = "/sequence/region/human/{}:{}-{}".format(self.variant.coordinates.chromosome, start, start)
                r = requests.get(ensembl_server+ext, headers={ "Content-Type" : "text/plain"})
                r.raise_for_status()
                ref = "{}{}".format(r.text, self.variant.coordinates.reference_bases)
                if self.variant.coordinates.variant_bases == None or self.variant.coordinates.variant_bases == '-' or self.variant.coordinates.variant_bases == '':
                    alt = r.text
                else:
                    alt = "{}{}".format(r.text, self.variant.coordinates.variant_bases)
        else:
            start = self.variant.coordinates.start
            ref = self.variant.coordinates.reference_bases
            alt = self.variant.coordinates.variant_bases
        return (start, ref, alt)

    def alt_record(self, alt):
        if self.variant.is_insertion:
            return vcfpy.Substitution(type_="INS", value=alt)
        elif self.variant.is_deletion:
            return vcfpy.Substitution(type_="DEL", value=alt)
        else:
            if len(alt) == 1:
                return vcfpy.Substitution(type_="SNV", value=alt)
            else:
                return vcfpy.Substitution(type_="MNV", value=alt)

    def csq_alt(self):
        if self.variant.coordinates.reference_build != 'GRCh37':
            return
        if self.variant.is_insertion:
            if not self.variant.coordinates.representative_transcript:
                return
            else:
                return self.variant.coordinates.variant_bases
        elif self.variant.is_deletion:
            if not self.variant.coordinates.representative_transcript:
                return
            else:
                return "-"
        else:
            return self.variant.coordinates.variant_bases

    def hgvs_c(self):
        if self.variant.coordinates.representative_transcript:
            hgvs_cs = [e for e in self.variant.hgvs_expressions if (':c.' in e) and (self.variant.coordinates.representative_transcript in e)]
            return hgvs_cs[0] if len(hgvs_cs) == 1 else ''
        else:
            return ''

    def hgvs_p(self):
        if self.variant.coordinates.representative_transcript:
            hgvs_ps = [e for e in self.variant.hgvs_expressions if (':p.' in e) and (self.variant.coordinates.representative_transcript in e)]
            return hgvs_ps[0] if len(hgvs_ps) == 1 else ''
        else:
            return ''

    def sanitized_name(self):
        name = self.variant.name
        regex = re.compile(r"^([A-Z]+)([0-9]+)(=)(.*)$")
        match = regex.match(name)
        if match is not None:
            name = "".join([match.group(1), match.group(2), match.group(1), match.group(4)])
        return name

    def csq(self, include_status=None):
        if self.csq_alt() is None:
            return []
        else:
            csq = []
            for mp in self.variant.molecular_profiles:
                for evidence in mp.evidence:
                    if include_status is not None and evidence.status not in include_status:
                        continue
                    csq.append('|'.join([
                        self.csq_alt(),
                        '&'.join(map(lambda t: t.name, self.variant.variant_types)),
                        self.variant.gene.name,
                        str(self.variant.gene.entrez_id),
                        'transcript',
                        str(self.variant.coordinates.representative_transcript),
                        self.hgvs_c(),
                        self.hgvs_p(),
                        self.sanitized_name(),
                        str(self.variant.id),
                        '&'.join(self.variant.variant_aliases),
                        "https://civicdb.org/links/variants/{}".format(self.variant.id),
                        mp.sanitized_name(),
                        str(mp.id),
                        '&'.join(mp.aliases),
                        "https://civicdb.org/links/molecular-profiles/{}".format(mp.id),
                        '&'.join(self.variant.hgvs_expressions),
                        str(self.variant.allele_registry_id),
                        '&'.join(self.variant.clinvar_entries),
                        str(mp.molecular_profile_score),
                        "evidence",
                        str(evidence.id),
                        "https://civicdb.org/links/evidence/{}".format(evidence.id),
                        "{} ({})".format(evidence.source.citation_id, evidence.source.source_type),
                        str(evidence.variant_origin),
                        evidence.status,
                        str(evidence.significance or ''),
                        str(evidence.evidence_direction or ''),
                        evidence.disease.name if evidence.disease is not None else "",
                        '&'.join([str(therapy) for therapy in evidence.therapies]),
                        str(evidence.therapy_interaction_type or ""),
                        '&'.join(["{} (HPO ID {})".format(phenotype.name, phenotype.hpo_id) for phenotype in evidence.phenotypes]),
                        evidence.evidence_level,
                        str(evidence.rating),
                        "",
                        "",
                        "",
                        "",
                        "",
                    ]))
                for assertion in mp.assertions:
                    if include_status is not None and assertion.status not in include_status:
                        continue
                    csq.append('|'.join([
                        self.csq_alt(),
                        '&'.join(map(lambda t: t.name, self.variant.variant_types)),
                        self.variant.gene.name,
                        str(self.variant.gene.entrez_id),
                        'transcript',
                        str(self.variant.coordinates.representative_transcript),
                        self.hgvs_c(),
                        self.hgvs_p(),
                        self.sanitized_name(),
                        str(self.variant.id),
                        '&'.join(self.variant.variant_aliases),
                        "https://civicdb.org/links/variants/{}".format(self.variant.id),
                        mp.sanitized_name(),
                        str(mp.id),
                        '&'.join(mp.aliases),
                        "https://civicdb.org/links/molecular-profiles/{}".format(mp.id),
                        '&'.join(self.variant.hgvs_expressions),
                        str(self.variant.allele_registry_id),
                        '&'.join(self.variant.clinvar_entries),
                        str(mp.molecular_profile_score),
                        "assertion",
                        str(assertion.id),
                        "https://civicdb.org/links/assertion/{}".format(assertion.id),
                        "",
                        str(assertion.variant_origin),
                        assertion.status,
                        assertion.significance,
                        assertion.assertion_direction,
                        str(assertion.disease),
                        '&'.join([str(therapy) for therapy in assertion.therapies]),
                        str(assertion.therapy_interaction_type or ''),
                        "",
                        "",
                        "",
                        "&".join([acmg_code.code for acmg_code in assertion.acmg_codes]),
                        str(assertion.amp_level or ''),
                        assertion.format_nccn_guideline(),
                        str(assertion.fda_regulatory_approval or ''),
                        str(assertion.fda_companion_test or ''),
                    ]))
            return csq
